use serde::{Deserialize, Serialize};

use crate::config;

#[derive(Debug, Serialize, Deserialize)]
pub struct Claim {
    sub: i64, // 用户ID
    exp: usize, // Token过期时间戳
}

pub fn generate_token(sub: i64) -> String {
    let claim = Claim {
        sub,
        exp: config!().jwt.expires,
    };
    generate(claim)
}

pub fn generate_refresh_token(sub: i64) -> String {
    let claim = Claim {
        sub,
        exp: config!().jwt.refresh_expires,
    };
    generate(claim)
}

fn generate(claim: Claim) -> String {
    let token = jsonwebtoken::encode(
        &jsonwebtoken::Header::default(),
        &claim,
        &jsonwebtoken::EncodingKey::from_secret(config!().jwt.secret.as_bytes()),
    );
    token.unwrap_or_else(|e| {
        tracing::error!(error =?e, "生成Token失败");
        "".to_string()
    })
}

pub fn verify_token(token: &str) -> Result<Claim, jsonwebtoken::errors::Error> {
    jsonwebtoken::decode::<Claim>(
        token,
        &jsonwebtoken::DecodingKey::from_secret(config!().jwt.secret.as_bytes()),
        &jsonwebtoken::Validation::default(),
    )
    .map(|data| data.claims)
}